use axum::{async_trait, body::Body, extract::{FromRequest, FromRequestParts, Query}};
use http::Request;
use i18n::{message, message_ids::MessageId};
use validator::Validate;

use crate::utils::request_util;

use super::response::ResErr;

pub struct QueryValidator<T>(pub T);

#[async_trait]
impl<S, T> FromRequest<S> for QueryValidator<T>
where
    S: Send + Sync,
    T: Validate,
    Query<T>: FromRequestParts<S>,
{
    type Rejection = ResErr;

    async fn from_request(req: http::Request<Body>, state: &S) -> Result<Self, Self::Rejection> {
        // let context = req.extensions().get().unwrap();
        
        let (parts, body) = req.into_parts();
        let header = &parts.headers;
        let query = Query::<T>::from_request(Request::from_parts(parts.clone(), body), state).await;
        let lang_tag = request_util::get_lang_tag(header);

        if let Ok(Query(data)) = query {
            match super::validator::validate_params(&data, &lang_tag) {
                Ok(_) => Ok(QueryValidator(data)),
                Err(err) => Err(err),
            }
        } else {
            let err = Err(ResErr::params(message!(&lang_tag, MessageId::InvalidParams)));
            err
        }
    }
}